function MessageService(ScreenService, MetaDataLoader, ZHPromise) {
	var self = this;

	var messageCodesPromise = MetaDataLoader.loadMessageByCode();

	this.getMessageByCode = function(messageCode) {
		return messageCodesPromise.then(function getMessage(allMessages) {
			var message = allMessages[messageCode];
			return message || ZHPromise.reject("Message not found!");
		});
	};

	this.showMessageByCode = function(messageCode, messageParameters, messageConfig) {
		return self.getMessageByCode(messageCode).then(
			function showMessageByCode(message) {
				var type = messageConfig && messageConfig.type;
				var icon = messageConfig && messageConfig.icon;

				return ScreenService.showMessage(message, type, icon, messageParameters);
			}
		);
	};

	this.confirmMessageByCode = function(messageCode, messageParameters, messageConfig) {
		return self.getMessageByCode(messageCode).then(
			function confirmMessage(message) {
				var type = messageConfig && messageConfig.type;
				var icon = messageConfig && messageConfig.icon;
				var success = messageConfig && messageConfig.success;
				var fail = messageConfig && messageConfig.fail;

				return ScreenService.confirmMessage(message, type, success, fail, icon, messageParameters);
			}
		);
	};

	this.showCustomDialogByCode = function(messageCode, messageParameters, dialogConfig) {
		return self.getMessageByCode(messageCode).then(
			function showCustomDialogByCode(message) {
				var type = dialogConfig && dialogConfig.type;
				var icon = dialogConfig && dialogConfig.icon;
				var actions = dialogConfig && dialogConfig.actions;

				return ScreenService.showCustomDialog(message, type, icon, actions, messageParameters);
			}
		);
	};

	this.notificationMessageByCode = function(messageCode, messageParameters, messageConfig) {
		return self.getMessageByCode(messageCode).then(
			function notificationMessageByCode(message) {
				var type = messageConfig && messageConfig.type;
				var time = messageConfig && messageConfig.time;

				ScreenService.notificationMessage(message, type, time, messageParameters);
			}
		);
	};

	this.alertNotificationByCode = function(messageCode, time, messageParameters) {
		return self.getMessageByCode(messageCode).then(
			function alertNotificationByCode(message) {

				ScreenService.alertNotification(message, time, messageParameters);
			}
		);
	};

	this.successNotificationByCode = function(messageCode, time, messageParameters) {
		return self.getMessageByCode(messageCode).then(
			function successNotificationByCode(message) {

				ScreenService.successNotification(message, time, messageParameters);
			}
		);
	};

	this.errorNotificationByCode = function(messageCode, messageParameters) {
		return self.getMessageByCode(messageCode).then(
			function errorNotificationByCode(message) {

				ScreenService.errorNotification(message, messageParameters);
			}
		);
	};
}