describe("Widget", function() {
	var zhPromise = new ZHPromise(Q);
	var localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine = {
		requestMetaData: function() {
			return zhPromise.when({});
		}
	};
	var memoryStorageEngine = {};
	var widget = null;
	/*jslint evil: true */
	var eventHandler = new EventHandler({}, {}, console);
	var methodBuilder = new MethodBuilder({}, {
		$eval: function(a, b) {
			return eval(a, b);
		}
	});
	var eventEngine = new EventEngine(new EventFactory(methodBuilder, {}),
		eventHandler, {
			updateTemplate: function() {}
		}, {
			i18n: function(text) {
				return text;
			}
		}, {
			logEvent: function() {}
		}
	);
	var aclEngine = new AclEngine();
	var screenService = {
		isPopupOpen: function() {
			return false;
		},
		closeSwipe: function() {}
	};
	var metaDataFactory = new MetaDataFactory(localStorageEngine, requestDataSourceEngine,
		SQLStorageStrategy, restEngine, eventEngine, screenService, aclEngine, {
			project: {}
		}, new ZhUtil({}), zhPromise, {
			getDataSourceLazy: function() {}
		}, {}, {}, {
			registerFeatureUsage: function() {}
		}, memoryStorageEngine);
	var baseContainer = function() {
		return {
			"widgets": [{
				"events": [{
					"name": "WidgetBeforeSelectAll",
					"code": ";"
				}, {
					"name": "WidgetAfterSelectAll",
					"code": ";"
				}],
				"dataSource": {
					"name": "/test",
					"sql": true,
					"data": [{}, {}, {}, {}]
				},
				"fields": [{
					"name": "name",
					"repeatOnNew": true,
					"defaultValue": 'nobody'
				}, {
					"name": "age",
					"repeatOnNew": false,
				}, {
					"name": "title",
					"repeatOnNew": true,
					"defaultValue": "Title d",
				}],
				"active": true,
				"actions": [{
					"name": "teste",
					"showAsAction": "edit_edit"
				}, {
					"name": "teste",
					"showAsAction": "edit"
				}, {
					"name": "teste",
					"showAsAction": "never"
				}, {
					"name": "teste",
					"showAsAction": "never|edit"
				}]
			}, {
				"dataSource": {
					"name": "/test2",
					"localStorage": true,
					"data": [{}, {}, {}, {}, {}]
				},
				"fields": [],
				"active": true,
				"widgets": [{
					"fields": [],
					"active": true,
					"dataSource": {
						"name": "/test2",
						"rest": true
					}
				}],
				"actions": [{
					"showAsAction": "checked_rows"
				}]
			}, ]
		};
	};
	beforeEach(function() {
		container = metaDataFactory.containerFactory(baseContainer());
		widget = container.widgets[0];
		widget.restoreDefaultMode();
	});
	it("Method select all should mark all items as __selected on Container", function() {
		container.selectAll();
		container.widgets[0].dataSource.data.forEach(function(item) {
			expect(item.__isSelected).toEqual(true);
		});
		expect(container.widgets[0].getCheckedRows().length).toEqual(4);
		expect(container.widgets[0].dataSource.checkedRows.length).toEqual(4);
		expect(container.widgets[1].dataSource.checkedRows.length).toEqual(0);

		container.unSelectAll();
		expect(container.widgets[0].dataSource.checkedRows.length).toEqual(0);
	});
	it("Method select all should mark all items as __selected on Widget", function() {
		container.widgets[0].selectAll();
		container.widgets[0].dataSource.data.forEach(function(item) {
			expect(item.__isSelected).toEqual(true);
		});
		expect(container.widgets[0].getCheckedRows().length).toEqual(4);
		expect(container.widgets[0].dataSource.checkedRows.length).toEqual(4);
		expect(container.widgets[1].dataSource.checkedRows.length).toEqual(0);

		container.widgets[0].unSelectAll();
		expect(container.widgets[0].dataSource.checkedRows.length).toEqual(0);
	});

	it("Method select all should mark all items as __selected on Container with index", function() {
		container.selectAll(1);
		container.widgets[1].dataSource.data.forEach(function(item) {
			expect(item.__isSelected).toEqual(true);
		});
		expect(container.widgets[0].getCheckedRows().length).toEqual(0);
		expect(container.widgets[0].dataSource.checkedRows.length).toEqual(0);
		expect(container.widgets[1].dataSource.checkedRows.length).toEqual(5);

		container.unSelectAll(1);
		expect(container.widgets[1].dataSource.checkedRows.length).toEqual(0);
	});
	it("Method select all should mark all items as __selected on Container with object", function() {
		container.selectAll(container.widgets[0]);
		container.widgets[0].dataSource.data.forEach(function(item) {
			expect(item.__isSelected).toEqual(true);
		});
		expect(container.widgets[0].getCheckedRows().length).toEqual(4);
		expect(container.widgets[0].dataSource.checkedRows.length).toEqual(4);
		expect(container.widgets[1].dataSource.checkedRows.length).toEqual(0);

		container.unSelectAll(container.widgets[0]);
		expect(container.widgets[0].dataSource.checkedRows.length).toEqual(0);
	});
	it("Method beforeSelectAll should be called before all items be selected", function() {
		var numberBefore = 100000;
		var numberAfter = -1;
		container.widgets[0].beforeSelectAll(function() {
			numberBefore = container.widgets[0].getCheckedRows().length;
		});
		container.widgets[0].afterSelectAll(function() {
			numberAfter = container.widgets[0].getCheckedRows().length;
		});
		container.selectAll(container.widgets[0]);

		expect(numberBefore).toEqual(0);
		expect(numberAfter).toEqual(4);
	});
	//
	it("Method toogle should toogle selection", function() {
		expect(container.widgets[0].getCheckedRows().length).toEqual(0);
		container.toogleSelection(container.widgets[0]);
		expect(container.widgets[0].getCheckedRows().length).toEqual(4);

		expect(container.widgets[1].getCheckedRows().length).toEqual(0);
		container.toogleSelection(container.widgets[1]);

		expect(container.widgets[1].getCheckedRows().length).toEqual(5);
		container.toogleSelection(container.widgets[1]);

		expect(container.widgets[1].getCheckedRows().length).toEqual(0);
	});
	/*
	 * @todo talk with rhalander about commented tests
	 */
	it("Method activate should inactive other widgets and active only current Widget", function() {
		container.widgets[0].activate();
		expect(container.widgets[0].active).toEqual(true);
		expect(container.widgets[1].active).toEqual(false);
		container.widgets[1].activate();
		//expect(container.widgets[1].lastActiveWidget).toEqual(container.widgets[0]);
		container.widgets[1].restoreLastActiveWidget();
	});

	it("Method activate should set the widget as its parent currentWidget if it has one", function() {
		widget.parent = {};
		widget.onActivate = function() {};

		widget.activate();
		expect(widget.parent.currentWidget).toEqual(widget);
	});

	it("Method activate should set the widget as its parent currentWidget only if showAsTab is true", function () {
		widget.showAsTab = false;
		widget.parent = {};
		widget.onActivate = function() {};

		widget.activate();
		expect(widget.parent.currentWidget).toBeUndefined();
	});

	/*
	 * @todo talk with rhalander about commented tests
	 */

	/*it("When active a widget and then active other and then active other one, when restoreLastActiveWidget the acitve widget is the secund widget, if call restoreLastActiveWidget again the active widget will be the first widget",
		function () {
			container.widgets[0].activate();
			container.widgets[1].activate();
			container.widgets[1].widgets[0].activate();
			expect(container.getActiveWidget()).toBe(container.widgets[1].widgets[0]);

			container.widgets[1].widgets[0].restoreLastActiveWidget();
			expect(container.getActiveWidget()).toBe(container.widgets[1]);

			container.widgets[1].restoreLastActiveWidget();
			expect(container.getActiveWidget()).toBe(container.widgets[0]);
		});
*/
	it("Get actions on Widget", function() {
		expect(container.widgets[0].getAction('teste').name).toBe("teste");
	});

	it("WidgetFactory without container should throw exception!", function() {
		expect(function() {
			metaDataFactory.widgetFactory({
				fields: [],
				actions: []
			});
		}).toThrow();
		expect(function() {
			metaDataFactory.widgetFactory({
				fields: [],
				actions: [],
				container: {}
			});
		}).not.toThrow();
	});

	it("Container getActionsToFilter should return all actions", function() {
		expect(widget.getActionsToFilter().length).toEqual(4);
	});

	it("widget getActionsToFilter should return actions with visibility using | as a separator", function() {
		expect(widget.getActionsByVisibility("edit").length).toEqual(2);
	});

	it("widget getActionsToFilter should return actions with visibility using with array parameter", function() {
		expect(widget.getActionsToFilter(["edit", "never"]).length).toEqual(3);
	});

	it("widget getActionsToFilter should return actions with visibility using with array parameter only of the active Widget", function() {
		widget.active = false;
		expect(widget.getActionsToFilter(["edit", "never"]).length).toEqual(3);
	});

	it("widget getActionsByVisibility should return actions with visibility in inactive widgets", function() {
		widget.active = false;
		widget.changeMode('edit');
		expect(widget.getActionsByVisibility("edit").length).toEqual(3);
	});

	it("widget getActionsByVisibility should return actions with visibility and visibility_$mode", function() {
		widget.active = true;
		widget.changeMode('edit');
		expect(widget.getActionsByVisibility("edit").length).toEqual(3);
	});
	it("widget isNew should be true when call newRow", function() {
		widget.active = true;
		widget.newRow();
		expect(widget.isNewRow()).toEqual(true);
	});
	it("widget isNew should be true when currenRow is undefined", function() {
		widget.active = true;
		widget.currentRow = undefined;
		expect(widget.isNewRow()).toEqual(false);
	});
	it("widget isNew should be true when currenRow is null", function() {
		widget.active = true;
		widget.currentRow = null;
		expect(widget.isNewRow()).toEqual(false);
	});
	it("widget repeatOnNew should be repeat values to the new row from oldRow and override the defaultValue, if the field has value on the row", function() {
		widget.active = true;
		widget.currentRow = {
			name: "Mateus",
			age: 26,
			title: undefined
		};
		widget.newRow();
		expect(widget.currentRow.name).toEqual("Mateus");
		expect(widget.currentRow.age).not.toEqual(26);
		expect(widget.currentRow.title).toEqual("Title d");
	});
	it("widget constructor should apply default properties", function() {
		var defaultValues = widget.getDefaultValues();
		var newWidget = new Widget(screenService, new ZhUtil({}), eventEngine);

		Object.keys(defaultValues).forEach(function(key) {
			if (Util.isFunction(defaultValues[key])) {
				expect(defaultValues[key].toString()).toEqual(newWidget[key].toString());
			} else {
				expect(defaultValues[key]).toEqual(newWidget[key]);
			}
		});

		expect(newWidget.formTemplate).toEqual(defaultValues.templateToForm);
	});

	it("widget isLazyLoad should return if the dataSource is using lazyLoad mode", function() {
		widget.dataSource = {
			lazyLoad: true
		};

		expect(widget.isLazyLoad()).toBeTruthy();
	});

	it("Method hasActive should return true if any of its children is active", function() {
		widget.widgets = [];
		widget.widgets.push(metaDataFactory.widgetFactory({
			active: true,
			container: {}
		}));
		widget.active = false;


		var active = widget.hasActive();
		expect(active).toBeTruthy();
	});

	it("Method moveToFirst should set the widget currentRow to the dataSource first item", function() {
		widget.dataSource = metaDataFactory.dataSourceFactory({
			data: [{
				prop: "WOWIMHERE"
			}]
		});

		widget.moveToFirst();
		expect(widget.currentRow.prop).toEqual(widget.dataSource.data[0].prop);
		expect(widget.currentRow.__index).toEqual(0);
	});

	it("Method moveToNext should go to the next dataSource item", function() {
		widget.dataSource = metaDataFactory.dataSourceFactory({
			data: [{
				prop: "WOWIMHERE"
			}, {
				prop: "WOWSECOND"
			}]
		});

		widget.moveToFirst();
		widget.moveToNext();
		expect(widget.currentRow.prop).toEqual(widget.dataSource.data[1].prop);
		expect(widget.currentRow.__index).toEqual(1);
	});

	it("Method moveToNext should go to the previous dataSource item", function() {
		widget.dataSource = metaDataFactory.dataSourceFactory({
			data: [{
				prop: "WOWIMHERE"
			}, {
				prop: "WOWSECOND"
			}]
		});

		widget.moveToFirst();
		widget.moveToNext();
		widget.moveToPrev();
		expect(widget.currentRow.prop).toEqual(widget.dataSource.data[0].prop);
		expect(widget.currentRow.__index).toEqual(0);
	});

	it("Method getParent should return the widget parent or its container", function() {
		var container = {
			id: "151515116515145151"
		};
		widget.container = container;
		widget.parent = null;

		expect(widget.getParent().id).toEqual(container.id);
		widget.container = null;
		widget.parent = container;
		expect(widget.getParent().id).toEqual(container.id);
	});

	it("Method list should change the widget and its parent mode and should change some other properties", function() {
		widget.list(true);

		expect(widget.editing).toBeFalsy();
		expect(widget.showForm).toBeFalsy();
		expect(widget.formTemplate).toEqual(widget.templateToView);
		expect(widget.getMode()).toEqual(ViewMode.LIST);
		expect(widget.container.getMode()).toEqual(ViewMode.LIST);
	});

	it("Method view should change the widget and its parent mode and should change some other properties", function() {
		widget.view();

		expect(widget.editing).toBeFalsy();
		expect(widget.showForm).toBeTruthy();
		expect(widget.formTemplate).toEqual(widget.templateToView);
		expect(widget.getMode()).toEqual(ViewMode.VIEW);
		expect(widget.container.getMode()).toEqual(ViewMode.VIEW);
	});

	it("Method saveRow should throw if the row doen't exist", function() {
		widget.currentRow = null;

		expect(function() {
			widget.saveRow();
		}).toThrow(new Error("Current row is undefined. Unable to save row."));
	});

	it("Method saveRow should change the currentRow state to changed", function() {
		widget.currentRow = {};

		widget.saveRow();
		expect(widget.currentRow.changed).toBeTruthy();
	});

	it("Method reload should call refreshCurrentRow when in VIEW mode and return a promise", function(done){
		widget.mode = ViewMode.VIEW;
		widget.dataSource = metaDataFactory.dataSourceFactory({
			data: [{
				prop: "WOWIMHERE"
			}, {
				prop: "WOWSECOND"
			}]
		});
		//getVO gets called inside refreshCurrentRow()
		spyOn(widget.dataSource,'getVO');
		widget.moveToFirst();
		widget.reload().then(function(){
			expect(widget.dataSource.getVO).toHaveBeenCalled();
			done();
		});
	});

	it("Method reload should not call refreshCurrentRow when in VIEW mode and with shouldRefreshCurrentRow flagged false. Also return a promise" , function(done){
		widget.mode = ViewMode.VIEW;
		widget.dataSource = metaDataFactory.dataSourceFactory({
			data: [{
				prop: "WOWIMHERE"
			}, {
				prop: "WOWSECOND"
			}]
		});
		spyOn(widget.dataSource,'getVO');
		widget.moveToFirst();
		widget.reload(null, false).then(function(){
			expect(widget.dataSource.getVO).not.toHaveBeenCalled();
			done();
		});
	});

	it("Method reload should call dataSource.load when not in VIEW mode" , function(done){
		widget.mode = ViewMode.LIST;
		widget.dataSource = metaDataFactory.dataSourceFactory({
			data: [{
				prop: "WOWIMHERE"
			}, {
				prop: "WOWSECOND"
			}]
		});
		spyOn(widget.dataSource,'load').and.callThrough();
		widget.moveToFirst();
		widget.reload().then(function(){
			expect(widget.dataSource.load).toHaveBeenCalled();
			done();
		});
	});
});