describe("WidgetValidation", function() {
	var localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine = {
		requestMetaData: function() {
			return Q.when({});
		}
	};
	var validateFunction;

	var widget = null;
	var eventHandler = new EventHandler({}, {}, console);
	var methodBuilder = new MethodBuilder({}, {
		$eval: function(a, b) {}
	});

	var eventEngine = new EventEngine(new EventFactory(methodBuilder, {}),
		eventHandler, {
			updateTemplate: function() {}
		}, {
			i18n: function(text) {
				return text;
			}
		}, {
			logEvent: function() {}
		}
	);

	var aclEngine = new AclEngine();
	var screenService = {
		isPopupOpen: function() {
			return false;
		},
		closeSwipe: function() {}
	};

	var validateEngine = {
		getMappedValidations: function() {
			return {
				required: function(value, validationAttrs) {
					var isDefined = !!value || Util.isNumber(value);
					return {
						"valid": !!(isDefined && (!Array.isArray(value) || value.length > 0)),
						"message": validationAttrs.message || "Required field"
					};
				}
			};
		}
	};

	var metaDataFactory = new MetaDataFactory(localStorageEngine, requestDataSourceEngine,
		SQLStorageStrategy, restEngine, eventEngine, screenService, aclEngine, {
			project: {}
		}, new ZhUtil({}), Q, {
			getDataSourceLazy: function() {}
		}, {}, {}, {
			registerFeatureUsage: function() {}
		}, {}, validateEngine);

	var baseContainer = function() {
		return {
			"widgets": [{
				"events": [{
					"name": "WidgetBeforeSelectAll",
					"code": ";"
				}, {
					"name": "WidgetAfterSelectAll",
					"code": ";"
				}],
				"dataSource": {
					"name": "/test",
					"sql": true,
					"data": [{}, {}, {}, {}]
				},
				"fields": [{
					"name": "name",
					"repeatOnNew": true,
					"defaultValue": 'nobody'
				}, {
					"name": "age",
					"repeatOnNew": false,
				}, {
					"name": "title",
					"repeatOnNew": true,
					"defaultValue": "Title d",
				}, {
					"name": "validate",
					"repeatOnNew": true,
					"defaultValue": "Title d",
					"validations": {
                        "required": true
                    }
				}],
				"active": true,
				"actions": [{
					"name": "teste",
					"showAsAction": "edit_edit"
				}, {
					"name": "teste",
					"showAsAction": "edit"
				}, {
					"name": "teste",
					"showAsAction": "never"
				}, {
					"name": "teste",
					"showAsAction": "never|edit"
				}]
			}, {
				"dataSource": {
					"name": "/test2",
					"localStorage": true,
					"data": [{}, {}, {}, {}, {}]
				},
				"fields": [],
				"active": true,
				"widgets": [{
					"fields": [],
					"active": true,
					"dataSource": {
						"name": "/test2",
						"rest": true
					}
				}],
				"actions": [{
					"showAsAction": "checked_rows"
				}]
			}]
		};
	};

	beforeEach(function() {
		container = metaDataFactory.containerFactory(baseContainer());
		widget = container.widgets[0];
		widget.restoreDefaultMode();
	});

	it("Should test if widget is valid with names and boolean return", function(){
		var invalidFields;
		var callback = function(response){
			invalidFields = response[3][0].errorMessage;
		};
		expect(widget.isValid(callback)).toBe(false);
		expect(invalidFields).toEqual("Required field");
	});

	it("Should test if widget is valid with message return", function(){
		var invalidFields;
		var callback = function(response){
			invalidFields = response;
		};

		widget.isValidWithMessage(callback);
		expect(invalidFields).toEqual("Error in field 'validate' with message - Required field.\n");
	});
});