describe("ListController", function() {
	var listController;
	var metaDataFactory;
	var zhPromise;
	var $scope;
	var $rootScope;
	var screenService;
	var container;
	var widget;
	window.templateUrls = [{
		package: "default",
		baseUrl: "../../../frontend/templates/dyman/"
	}];
	var baseContainer = ZhHelper.baseContainerWithTwoWidgets;
	var baseDataSourceData = ZhHelper.getTestDataSourceDataWithNames;
	var baseFields = ZhHelper.baseFields;

	var baseWidgetActions = function() {
		return [{
			"name": "NEW-LEFT",
			"aclIsVisible": true,
			"isVisible": true,
			"icon": "search",
			"showAsAction": "grid-left",
			"condition": "true",
			"tooltip": "New",
			"events": [],
			"id": "1"
		}, {
			"name": "NEW-RIGHT",
			"aclIsVisible": true,
			"isVisible": true,
			"icon": "search",
			"showAsAction": "grid-right",
			"condition": "true",
			"tooltip": "New",
			"events": [],
			"id": "1"
		}];
	};

	beforeEach(function() {
		module(function($provide) {
			//Providing a new $timeout because $timeout.flush is not flushing pending $timeout calls
			$provide.constant("$timeout", setTimeout);
			$provide.constant("$q", Q);
			$provide.service('I18nEngine', function() {
				var words = {
					"en_us": {
						"THE UNIQUE VALUE OF HU": "WOW TRANSLATED WORD!"
					}
 				};

				var language;
				this.setLanguage = function(_language) {
					language = _language;
					return Q.when(words[language] || {});
				};

				this.i18n = function(word) {
					return words[language] && words[language][word] || words.default && words.default[word] || word;
				};
			});
		});
		module("ZeedhiApp");
		module("ZeedhiServices");

		inject(function(_$rootScope_, $controller, _ScreenService_, _metaDataFactory_, _ZHPromise_) {
			metaDataFactory = _metaDataFactory_;
			screenService = _ScreenService_;
			zhPromise = _ZHPromise_;
			container = baseContainer();
			container.widgets[0].template = "widget/list.html";
			container.widgets[0].fields = [];
			container = metaDataFactory.containerFactory(container);
			var field = metaDataFactory.fieldFactory({name:"name",template:"calendar.html"});
			widget = container.widgets[0];
			widget.fields.push(field);
			widget.searchCriteria = {};
			widget.resetInfinityScroll = function() {
				return true;
			};

			$rootScope = _$rootScope_;
			$scope = _$rootScope_.$new();
			$scope.$parent = _$rootScope_;
			$scope.widget = widget;

			listController = $controller("ListController", {
				$scope: $scope
			});
		});
	});

	it("listController.widget: should return the $scope.widget or the $scope._currentPopup", function() {
		expect(listController.widget).toEqual(widget);
		$scope.widget = null;
		$scope._currentPopup = widget.widgets[0];
		expect(listController.widget).toEqual(widget.widgets[0]);
	});

	it("isTree: should return true if the widget template is tree/grid.html or tree/list.html, otherwise should return false", function() {
		widget.template = listController.getTreeTemplate();
		expect(listController.isTree(widget)).toBeTruthy();
		widget.template = listController.getTreeListTemplate();
		expect(listController.isTree(widget)).toBeTruthy();
		widget.template = listController.getGanttTemplate();
		expect(listController.isTree(widget)).toBeFalsy();
	});

	it("isGantt: should return true if the widget template is tree/gantt.html, otherwise should return false", function() {
		widget.template = listController.getGanttTemplate();
		expect(listController.isGantt(widget)).toBeTruthy();
		widget.template = listController.getTreeTemplate();
		expect(listController.isGantt(widget)).toBeFalsy();
	});

	it("isCrosstab: should return true if the widget template is crosstab.html, otherwise should return false", function() {
		widget.template = listController.getCrosstabTemplate();
		expect(listController.isCrosstab(widget)).toBeTruthy();
		widget.template = listController.getTreeTemplate();
		expect(listController.isCrosstab(widget)).toBeFalsy();
	});

	it("search: should set searched items __notSelectable to false", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.dataSource.data.forEach(function(data) {
			listController.search(data);
			expect(data.__notSelectable).toBeFalsy();
		});
	});

	it("search: should filter the values according to the widget.searchCriteria for any property of the data", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();
		widget.searchCriteria = { "_ALL": "Rodrigo" };
		var filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(8);
	});

	it("search: should work when using strict comparison", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();
		widget.searchCriteria = { "_ALL": "\"Rodrigo Cesar abc crazy\"" };
		var filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(1);
	});

	it("search: should filter the values when there are multiple words in the _ALL condition", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();
		widget.searchCriteria = { "_ALL": "Rodrigo dsf" };
		var filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(7);
	});

	it("search: should filter the values when there are multiple words in the _ALL condition and other fields conditions", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();
		widget.searchCriteria = { "_ALL": "Rodrigo dsf", hu: "12" };
		var filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(2);
	});

	it("search: should not search for this field if the field skipSearch is true, or isVisible or showOnList is false", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();
		widget.fields.forEach(function(field) {
			field.skipSearch = true;
		});
		widget.searchCriteria = { "_ALL": "Rodrigo" };
		var filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(0);

		widget.fields.forEach(function(field) {
			field.skipSearch = true;
			field.showOnList = false;
		});
		filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(0);

		widget.fields.forEach(function(field) {
			field.skipSearch = true;
			field.showOnList = true;
			field.isVisible = false;
		});
		filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(0);
	});

	it("search: should search from translations when field.translate is true", function() {
		screenService.setLanguage('en_us').then(function() {
			widget.dataSource.data = baseDataSourceData();
			widget.fields = baseFields();

			widget.fields[widget.fields.length - 1].translate = true;

			widget.searchCriteria = { "_ALL": "WOW TRANSLATED WORD!" };
			var filteredData = widget.dataSource.data.filter(listController.search);
			expect(filteredData.length).toBe(1);
			done();
		});
	});


	it("search: should return true if there is no searchCriteria or the seachList is empty", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();

		widget.searchCriteria = {};
		var filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(widget.dataSource.data.length);
	});

	it("search: should escape searches that have regex chars", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();
		widget.dataSource.data[0].name = "uabafet \\ ^ $ * + ? . ( ) something between | { } [ ] wow crazy";
		widget.searchCriteria = { name: "uabafet \\ ^ $ * + ? . ( ) | { } [ ]" };
		var filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(1);
	});

	it("clearSearchList: should clear the widget.searchCriteria", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();
		widget.searchCriteria = { "_ALL": "Rodrigo" };
		$scope.$apply();
		expect(widget.searchCriteria).toEqual({ "_ALL": "Rodrigo" });

		widget.clearSearchList();
		expect(widget.searchCriteria).toEqual({ });
	});

	it("Should add a method to clear the $scope's searchCriteria", function() {
		widget.dataSource.data = baseDataSourceData();
		widget.fields = baseFields();
		widget.searchCriteria = { "_ALL": "Rodrigo" };
		var filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(8);

		widget.clearSearchList();
		$scope.$apply();
		expect(widget.searchCriteria).toEqual({ });
		filteredData = widget.dataSource.data.filter(listController.search);
		expect(filteredData.length).toBe(widget.dataSource.data.length);
	});

	it("_searchWithPaginationReload: should reject if there is no field", function(done) {
		var mock = ZhHelper.buildMock();
		listController._searchWithPaginationReload(widget, function(searchPromise) {
			searchPromise
				.then(function() {
					mock.whenSucces();
					expect(mock.whenError).toHaveBeenCalled();
					done();
				})
				.catch(function() {
					mock.whenError();
					expect(mock.whenSuccess).not.toHaveBeenCalled();
					done();
				});
		});
	});

	it("_searchWithPaginationReload: should return the widget.reload promise", function(done) {
		var mock = ZhHelper.buildMock();
		widget.reload = function() {
			return zhPromise.when();
		};
		widget.fields = [{}];
		listController._searchWithPaginationReload(widget, function(searchPromise) {
			searchPromise
				.then(function() {
					mock.whenSuccess();
					expect(mock.whenError).not.toHaveBeenCalled();
					expect(mock.whenSuccess).toHaveBeenCalled();
					done();
				})
				.catch(function() {
					mock.whenError();
					expect(mock.whenError).not.toHaveBeenCalled();
					expect(mock.whenSuccess).toHaveBeenCalled();
					done();
				});
		});
	});

	it("deselectAll: should fire widget.unSelectAll and call dataSource clearCheckedRows if there is no event or it is not prevented default", function() {
		widget.unSelectAll = function() {};
		var clearCheckedRowsCallCount = 0;
		widget.dataSource.clearCheckedRows = function() {
			clearCheckedRowsCallCount++;
		};
		screenService.deselectAll(widget);
		//listController.deselectAll(widget);
		expect(clearCheckedRowsCallCount).toBe(1);
	});




	it("toggleAll: should call widget unSelectAll if the widget has checked rows", function() {
		var unSelectAllCallCount = 0;
		var updateCheckedRowsCallCount = 0;
		widget.hasCheckedRows = function() {
			return true;
		};
		widget.unSelectAll = function() {
			unSelectAllCallCount++;
		};
		widget.updateCheckedRows = function() {
			updateCheckedRowsCallCount++;
		};
		widget.groupedData = [{
			"__isSelected": true
		}];
		listController.toggleAll();
		expect(unSelectAllCallCount).toBe(1);
		expect(updateCheckedRowsCallCount).toBe(1);
	});

	it("toggleAll: should call widget selectAll if the widget has no checked row", function() {
		var selectAllCallCount = 0;
		var updateCheckedRowsCallCount = 0;
		widget.hasCheckedRows = function() {
			return false;
		};
		widget.selectAll = function() {
			selectAllCallCount++;
		};
		widget.updateCheckedRows = function() {
			updateCheckedRowsCallCount++;
		};
		listController.toggleAll();
		expect(selectAllCallCount).toBe(1);
		expect(updateCheckedRowsCallCount).toBe(1);
	});

	it("toggle: should trigger widget beforeUnselectRow if the row is selected", function() {
		var beforeUnselectRowCallCount = 0;
		widget.beforeUnselectRow = function() {
			beforeUnselectRowCallCount++;
		};
		var row = {
			__isSelected: true
		};
		listController.toggle(row);
		expect(beforeUnselectRowCallCount).toBe(1);
	});

	it("toggle: should trigger widget beforeSelectRow if the row is not selected if the event is not defaultPrevented", function() {
		var beforeSelectRowCallCount = 0;
		widget.beforeSelectRow = function() {
			beforeSelectRowCallCount++;
		};
		var row = {
			__isSelected: false
		};
		listController.toggle(row);
		expect(beforeSelectRowCallCount).toBe(1);
	});

	it("toggle: should trigger widget afterUnselectRow if the row is selected", function() {
		var afterUnselectRowCallCount = 0;
		widget.afterUnselectRow = function() {
			afterUnselectRowCallCount++;
		};
		var row = {
			__isSelected: true
		};
		widget.dataSource.checkedRows = [row];
		listController.toggle(row);
		expect(afterUnselectRowCallCount).toBe(1);
	});

	it("toggle: should trigger widget afterSelectRow if the row is not selected", function() {
		var afterselectRowCallCount = 0;
		widget.afterSelectRow = function() {
			afterselectRowCallCount++;
		};
		var row = {
			__isSelected: true
		};
		listController.toggle(row);
		expect(afterselectRowCallCount).toBe(1);
	});

	it("toggle: should set __isSelected to true if the row is in the dataSource checkedRows", function() {
		widget.beforeSelectRow = function() {};
		var row = {
			__isSelected: false
		};
		widget.dataSource.checkedRows = [];
		listController.toggle(row);
		expect(row.__isSelected).toBeTruthy();
	});

	it("toggle: should set __isSelected to false if the row is not in the dataSource checkedRows", function() {
		widget.beforeSelectRow = function() {};
		var row = {
			__isSelected: true
		};
		widget.dataSource.checkedRows = [row];
		listController.toggle(row);
		expect(row.__isSelected).toBeFalsy();
	});

	it("toggle: should trigger ui.removeFromFooter if there is no checkedRow", function() {
		var uiRemoveFromFooterCallCount = 0;
		widget.beforeSelectRow = function() {};
		var row = {
			__isSelected: true
		};
		$rootScope.$on("ui.removeFromFooter", function() {
			uiRemoveFromFooterCallCount++;
		});

		widget.dataSource.checkedRows = [];
		listController.toggle(row);
		expect(uiRemoveFromFooterCallCount).toBe(1);
	});

	it("$scope.openRow: should only return if the row has __group property", function() {
		var toggleCallCount = 0;
		var row = {
			__group: {}
		};
		listController.toggle = function() {
			toggleCallCount++;
		};
		$scope.openRow(row, widget);
		expect(toggleCallCount).toBe(0);
	});

	it("$scope.openRow: should toggle if the widget has allowMultiSelect or widget has checkedRows", function() {
		widget.allowMultiSelect = true;
		widget.getCheckedRows = function() {
			return [{}];
		};
		var toggleCallCount = 0;
		var row = {};
		listController.toggle = function() {
			toggleCallCount++;
		};
		$scope.openRow(row, widget);
		expect(toggleCallCount).toBe(1);
	});

	it("$scope.openRow: should set the widget currentRow if the widget has no checked row", function() {
		var toggleCallCount = 0;
		var row = {};
		$scope.openRow(row, widget);
		expect(widget.currentRow).toBe(row);
	});

	it("$scope.openRowInnerList: should delete the widget currentRow if the currentRow index is the passed row", function() {
		var toggleCallCount = 0;
		var row = {};
		widget.dataSource.data = [row];
		widget.setCurrentRow(row);
		$scope.openRowInnerList(row, widget);
		expect(widget.currentRow).not.toBeDefined();
	});

	it("$scope.openRowInnerList: should open the row if the currentRow is not selected", function() {
		var toggleCallCount = 0;
		var row = {};
		var secondRow = {};
		widget.dataSource.data = [row, secondRow];
		widget.setCurrentRow(secondRow);
		$scope.openRowInnerList(row, widget);
		expect(widget.currentRow).toEqual(row);
	});

	it("onLongTap: should call toggle if the widget allows selection on long tap", function() {
		var toggleCallCount = 0;
		widget.allowSelectOnLongTap = true;
		var row = {};
		listController.toggle = function() {
			toggleCallCount++;
		};
		listController.onLongTap(row, widget);
		expect(toggleCallCount).toBe(1);
	});

	it("onLongTap: should not call toggle if the widget does not allow selection on long tap", function() {
		var toggleCallCount = 0;
		widget.allowSelectOnLongTap = false;
		var row = {};
		listController.toggle = function() {
			toggleCallCount++;
		};
		listController.onLongTap(row, widget);
		expect(toggleCallCount).toBe(0);
	});

	it("ui.allowMultiSelect event: should set allowMultiSelect to the widget of the event", function() {
		$rootScope.$broadcast("ui.allowMultiSelect", widget);
		expect(listController.getAllowMultiSelect()).toBe(widget);
	});

	it("ui.disableMultiSelect event: should set allowMultiSelect to the widget of the event", function() {
		$rootScope.$broadcast("ui.disableMultiSelect", widget);
		expect(listController.getAllowMultiSelect()).toBeNull();
	});

	it("getGroupHeaderText: should return a text without the colon if there is no row.__groupLabel", function() {
		var row = {
			__groupValue: "value"
		};
		expect(listController.getGroupHeaderText(row)).toEqual("value");
	});

	it("getGroupHeaderText: should return a text without the colon if there is no row.__groupValue", function() {
		var row = {
			__groupLabel: "label"
		};
		expect(listController.getGroupHeaderText(row)).toEqual("label");
	});

	it("getGroupHeaderText: should return the group label concatenated to a colon and the group value", function() {
		var row = {
			__groupLabel: "label",
			__groupValue: "value"
		};
		expect(listController.getGroupHeaderText(row)).toEqual("label: value");
	});

	it("getGroupHeaderText: should an empty string if there is no row", function() {
		var row = null;
		expect(listController.getGroupHeaderText(row)).toEqual("");
	});

	it("visibleLeftActions and visibleRightActions: should change the row.__rightActions and row.__leftActions", function() {
		widget.dataSource.data = undefined;
		widget.template = "widget/grid.html";
		widget.actions = baseWidgetActions();
		$scope.$apply();
		var firstRow = {};
		widget.dataSource.data = [firstRow];
		$scope.buildGridActions();
		$scope.$apply();

		expect($scope.visibleRightActions).toBe(1);
		expect($scope.visibleLeftActions).toBe(1);
		expect(firstRow.__rightActions).toBeDefined();
		expect(firstRow.__rightActions.length).toBe(1);
		expect(firstRow.__leftActions).toBeDefined();
		expect(firstRow.__leftActions.length).toBe(1);
	});

	it("ui.allowMultiSelect event: should set allowMultiSelect to the widget of the event", function() {
		$rootScope.$broadcast("ui.allowMultiSelect", widget);
		expect(listController.getAllowMultiSelect()).toBe(widget);
	});

	it("ui.disableMultiSelect event: should set allowMultiSelect to the widget of the event", function() {
		$rootScope.$broadcast("ui.disableMultiSelect", widget);
		expect(listController.getAllowMultiSelect()).toBeNull();
	});

	it("visibleLeftActions and visibleRightActions: should change the row.__rightActions and row.__leftActions", function() {
		widget.dataSource.data = undefined;
		widget.template = "widget/grid.html";
		widget.actions = baseWidgetActions();
		$scope.$apply();
		var firstRow = {};
		widget.dataSource.data = [firstRow];
		$scope.buildGridActions();
		$scope.$apply();

		expect($scope.visibleRightActions).toBe(1);
		expect($scope.visibleLeftActions).toBe(1);
		expect(firstRow.__rightActions).toBeDefined();
		expect(firstRow.__rightActions.length).toBe(1);
		expect(firstRow.__leftActions).toBeDefined();
		expect(firstRow.__leftActions.length).toBe(1);
	});

	it("ui.allowMultiSelect event: should set allowMultiSelect to the widget of the event", function() {
		$rootScope.$broadcast("ui.allowMultiSelect", widget);
		expect(listController.getAllowMultiSelect()).toBe(widget);
	});

	it("ui.disableMultiSelect event: should set allowMultiSelect to the widget of the event", function() {
		$rootScope.$broadcast("ui.disableMultiSelect", widget);
		expect(listController.getAllowMultiSelect()).toBeNull();
	});

	it("visibleLeftActions and visibleRightActions: should change the row.__rightActions and row.__leftActions", function() {
		widget.dataSource.data = undefined;
		widget.template = "widget/grid.html";
		widget.actions = baseWidgetActions();
		$scope.$apply();
		var firstRow = {};
		widget.dataSource.data = [firstRow];
		$scope.buildGridActions();
		$scope.$apply();

		expect($scope.visibleRightActions).toBe(1);
		expect($scope.visibleLeftActions).toBe(1);
		expect(firstRow.__rightActions).toBeDefined();
		expect(firstRow.__rightActions.length).toBe(1);
		expect(firstRow.__leftActions).toBeDefined();
		expect(firstRow.__leftActions.length).toBe(1);

		widget.actions[0].condition = "false";
		widget.actions[1].condition = "false";
		widget.dataSource.data = [{}];
		$scope.buildGridActions();
		$scope.$apply();
		firstRow = widget.dataSource.data[0];

		expect($scope.visibleRightActions).toBe(0);
		expect($scope.visibleLeftActions).toBe(0);
		expect(firstRow.__rightActions).toBeDefined();
		expect(firstRow.__rightActions.length).toBe(0);
		expect(firstRow.__leftActions).toBeDefined();
		expect(firstRow.__leftActions.length).toBe(0);
	});

	it("changeOrder: should reset the widget order and set widget reverse to false if the widget.order is the column name", function() {
		widget.fields = baseFields();
		widget.order = widget.fields[0].name;
		widget.reverse = true;
		listController.changeOrder(widget.fields[0]);
		expect(widget.order).not.toBeDefined();
		expect(widget.reverse).toBeFalsy();
	});

	it("changeOrder: should set widget.reverse to true if the widget.reverse is false and the widget.order is the column name", function() {
		widget.fields = baseFields();
		widget.order = widget.fields[0].name;
		widget.reverse = false;
		listController.changeOrder(widget.fields[0]);
		expect(widget.order).toEqual(widget.fields[0].name);
		expect(widget.reverse).toBeTruthy();
	});

	it("stripeFooter: should return nothing if the widget has no widgets", function() {
		$scope.widget.widgets = null;
		expect(listController.stripeFooter()).toBeUndefined();
	});

	it("stripeFooter: should return the widget that has a stripe template", function() {
		var stripeWidget = {
			template: "templates/widget/stripe.html"
		};
		$scope.widget.widgets[0] = stripeWidget;
		expect(listController.stripeFooter()).toEqual(stripeWidget);
	});

	it("updateOrderByColumns: should not order by the widget groups if orderFromGroups is false", function() {
		widget.orderFromGroups = false;
		widget.groups = ["name"];
		listController.updateOrderByColumns(widget);
		expect(listController.getOrderByColumns()).toEqual([]);
	});

	it("updateOrderByColumns: should order by the widget groups", function() {
		widget.groups = [{
			name: "name"
		}];
		listController.updateOrderByColumns(widget);
		expect(listController.getOrderByColumns()).toEqual(["__proxy_name"]);
	});

	it("updateOrderByColumns: should order by widget.order", function() {
		widget.order = ["name", "value"];
		listController.updateOrderByColumns(widget);
		expect(listController.getOrderByColumns()).toEqual(["__proxy_name", "value"]);
	});

	it("updateOrderByColumns: should order by widget.order with fields", function() {
		widget.order = ["name", "value"];
		widget.fields.push({name:"name"});
		listController.updateOrderByColumns(widget);
		expect(listController.getOrderByColumns()).toEqual(["__proxy_name", "value"]);
	});


	it("updateOrderByColumns: should order by widget order with a minus sign if widget.reverse is true", function() {
		widget.order = "name";
		widget.reverse = true;
		listController.updateOrderByColumns(widget);
		expect(listController.getOrderByColumns()).toEqual(["-__proxy_name"]);
	});

	it("updateOrderByColumns: should not duplicate ordering", function() {
		widget.order = ["name", "name"];
		widget.reverse = true;
		listController.updateOrderByColumns(widget);
		expect(listController.getOrderByColumns()).toEqual(["-__proxy_name"]);
	});

	it("Should create watch on currentRow", function() {
		var $scope = {
			$watchCollection: function() {},
			$watch: function() {},
			widget: {
				getActionsByVisibility: function() {return [];},
				reload: function() {},
				actions: [],
				fields: []
			}
		};
		spyOn($scope, '$watch');
		spyOn($scope, '$watchCollection');

		inject(function($controller) {
			listController = $controller("ListController", {
				$scope: $scope
			});
		});

		expect($scope.$watch).toHaveBeenCalledTimes(2);
		expect($scope.$watchCollection).toHaveBeenCalledTimes(2);
		expect($scope.$watch).toHaveBeenCalledWith(jasmine.any(Function), jasmine.any(Function), true);
		expect($scope.$watch).toHaveBeenCalledWith(jasmine.any(Function), jasmine.any(Function), true);
	});

	it("Should not call reload on init when lazyLoad is true", function(done) {
		var widget;

		inject(function(_$rootScope_, $controller) {
			var container = baseContainer();
			container.widgets[0].template = "widget/list.html";
			container.widgets[0].fields = [];
			container.widgets[0].dataSource = {
				name: '/test',
				lazyLoad: true,
				localStorage: true
			};
			container = metaDataFactory.containerFactory(container);
			widget = container.widgets[0];

			var field = metaDataFactory.fieldFactory({name:"name",template:"calendar.html"});
			widget.fields.push(field);

			$rootScope = _$rootScope_;
			$scope = _$rootScope_.$new();
			$scope.$parent = _$rootScope_;
			$scope.widget = widget;

			listController = $controller("ListController", {
				$scope: $scope
			});

		});

		spyOn(listController, '_searchWithPaginationReload');
		$scope.$digest();

		setTimeout(function() {
			expect(listController._searchWithPaginationReload).not.toHaveBeenCalled();
			done();
		}, 1000);
	});
});