var KDSModule = angular.module("ZeedhiDirectives");

KDSModule.directive('zhKdsContainer', function($interval, ApplicationContext) {
	return {
		restrict: 'A',
		link: function($scope, element, attrs) {
			"use strict";

			var widget = $scope.widget;
			let watchers;

			$scope.$on('suspend', function () {
				watchers = $scope.$$watchers;
				$scope.$$watchers = [];
			});
		
			$scope.$on('resume', function () {
				if (watchers)
					$scope.$$watchers = watchers;
		
				watchers = void 0;
			});

			$interval(updateOrdersTimers, 1000, 0, false);

			function updateOrdersTimers () {
				widget.dataSource.data.forEach(function(order) {
					if(order.updateOrderTimer) {
						order.updateOrderTimer();
					}
				});
			}

			widget.bindPanelShortcutKeys = function() {
				ApplicationContext.ShortcutKeyService.bindShortcutsOrderPanel();
			};

			widget.orderDataSouce = function () {
				removeOrdersCompleted();
				widget.dataSource.data = removeOrders();
				// widget.dataSource.data = removeEmptyOrders();
				widget.dataSource.data = groupItemsExpedition(widget.dataSource.data); // Agrupa os Itens Iguais na expedição e montagem
				widget.dataSource.data.sort(sortDataSource);
				widget.dataSource.hasPriorityOrders = hasPriorityOrders();
				// Dar um update template se não rolar
			};

			widget.orderDataSourceByPositionTable = function () {
				widget.dataSource.data.sort(sortDataSourceByPositionTable);
			};

			widget.orderDataSourceByActionLabel = function () {
				widget.dataSource.data.sort(sortDataSourceByActionLabel);
			};

			function sortDataSource(a, b) {
				var orderDateA,orderDateB;
				orderDateA = chooseOrdenationTime(a, orderDateA);
				orderDateB = chooseOrdenationTime(b, orderDateB);

				if(orderDateA.isSame(orderDateB, 'second')){
					if(a.NRPEDIDOFOS == b.NRPEDIDOFOS){
						if(moment(a.DTACRPRIORTIME).isSame(moment(b.DTACRPRIORTIME), 'second')) {
							return a.orderKey > b.orderKey? 1:-1;
						} else {
							if(moment(a.DTACRPRIORTIME).isBefore(moment(b.DTACRPRIORTIME))) {
								return -1;
							} else {
								return 1;
							}
						}
					} else {
						return a.NRPEDIDOFOS > b.NRPEDIDOFOS? 1:-1;
					}
				} else {
					return orderDateA.isBefore(orderDateB, 'second') ? -1 : 1;
				}

			}

			function sortDataSourceByPositionTable (a, b) {
				if (a.NRPEDIDOFOS == b.NRPEDIDOFOS) {
					if(a.NRLUGARMESAIT == b.NRLUGARMESAIT){
						if(parseInt(a.NRTEMPOPROD) == parseInt(b.NRTEMPOPROD)){
							if(a.NRPRODCOMVEN){
								if(parseInt(a.NRPRODCOMVEN) < parseInt(b.NRPRODCOMVEN)){
									return -1;
								}else{
									return 1;
								}
							} else if (parseInt(a.NRSEQUITVEND < b.NRSEQUITVEND)){
								return -1;
							} else {
								return 0;
							}
						} else {
							return parseInt(a.NRTEMPOPROD) < parseInt(b.NRTEMPOPROD) ? 1 : -1;
						}						
					}else{
						return a.NRLUGARMESAIT > b.NRLUGARMESAIT? 1 : -1;
					}
				}else {
					return a.NRPEDIDOFOS > b.NRPEDIDOFOS? 1 : -1;
				}
			}

			function sortDataSourceByActionLabel (a, b) {
				/* ordena o array colocando os itens produzidos nas primeiras posições */
				let actionA = orderCanExpedite(a);
				let actionB = orderCanExpedite(b);
				let nrPedidoA = parseInt(a.NRPEDIDOFOS);
				let nrPedidoB = parseInt(b.NRPEDIDOFOS);

				if (actionA === actionB) {
					if (nrPedidoA === nrPedidoB) {
						return parseInt(a.NRTEMPOPROD) < parseInt(b.NRTEMPOPROD) ? 1 : -1;
					}
					return nrPedidoA > nrPedidoB ? 1 : -1;
				}

				return (actionA === true) ? -1 : 1;
			}

			function orderCanExpedite(order) {
				// Verifica se todos os itens estão produzidos
				let	canExpedite = order.items.every(function(item) {
					return item[widget.orderItemSelectField];
				});
				return canExpedite;
			}

			function chooseOrdenationTime(order, orderDate) {
				if (order.NRSEQLIBERADO || order.items[0] && (order.items[0].NRATRAPRODITPE && order.items[0].NRATRAPRODITPE !== 0)) {
					var NRMAIORTEMPO = order.NRMAIORTEMPO && widget.IDTIPOSETOR == 'P' ? order.NRMAIORTEMPO: 0;
					return moment(order.DTACRPRIORTIME).subtract(NRMAIORTEMPO, 'seconds');
				} else {
					return moment(order.DTACRPRIORTIME);
				}
			}

			function groupItemsExpedition (orders) {
				let allowGroup = localStorage.getItem('groupItemsExpedition') === 'true';
				let environment = ApplicationContext.environment.getLocalEnvironment();
				if (!allowGroup) {
					return orders;
				}

				for (let i = 0; i < orders.length; i++) {
					let order = orders[i];
					if (order && order.IDTIPOSETOR != 'P'){
						order.items = checkAndGroupItems(order.items);
					}
				}
				return orders;
			}
			// armazena a checkAndGroupItems no widget para ser chamada quando tranferir os itens de mesa
			widget.checkAndGroupItems = checkAndGroupItems;

			function checkAndGroupItems(items) {
				if (!items || items.length <= 1) {
					return items;
				}
			
				let groupedItens = [];
				let itemsToRemove = new Set();
			
				items.forEach((e) => {
					if (itemsToRemove.has(e)) {
						return;
					}
			
					let itensIguais = items.filter((element) => {
						return (
							element.CDSETOR === e.CDSETOR &&
							element.NRPEDIDOFOS === e.NRPEDIDOFOS &&
							element.CDSENHAPED === e.CDSENHAPED &&
							element.NRSEQPRODCOMKDS === e.NRSEQPRODCOMKDS &&
							element.CDPRODPROMOKDS === e.CDPRODPROMOKDS &&
							element.CDPRODUTO === e.CDPRODUTO &&
							element.DSLUGARMESAIT === e.DSLUGARMESAIT &&
							element.TXPRODPED === e.TXPRODPED &&
							((e.IDSITITPEDFOS !== 'C' && ['E', 'P', 'F'].includes(element.IDSITITPEDFOS)) ||
							 (e.IDSITITPEDFOS === 'C' && element.IDSITITPEDFOS === 'C'))
						);
					});
			
					if (itensIguais.length > 1) {
						/* jshint ignore:start */
						let groupedItem = { ...e }; // jshint ignore:line
						/* jshint ignore:end */
						let quantidade = itensIguais.reduce((total, it) => total + parseFloat(it.QTPRODPEFOS), 0);
			
						itensIguais.forEach(it => itemsToRemove.add(it));
						
						groupedItem.QTPRODPEFOS = quantidade;
						groupedItem.groupedItens = itensIguais;
						groupedItem[widget.orderItemSelectField] = itensIguais.every(item => item[widget.orderItemSelectField]);
						groupedItem[widget.orderItemStatusField] = itensIguais.every(it => it[widget.orderItemStatusField] === widget.itemDoneStatus) ? 'F' : 'E';
			
						groupedItens.push(groupedItem);
					}
				});
			
				items = items.filter(item => !itemsToRemove.has(item));
				items = items.concat(groupedItens);
			
				return items;
			}

			widget.updateKDSTemplate = function() {
				if(!$scope.$$phase) {
					$scope.$broadcast('suspend');
					$scope.$digest();
					$scope.$broadcast('resume');
				}
			};

			widget.getItemsArray = function(order){
				const sortItemsByComboOrder =  localStorage.getItem('sortItemsByComboOrder') || false;

				//TA VENDO ISSO AQUI? É UM CACHE NA MÃO
				if (order.LASTUPDATE != order.cacheLastUpdate) {
					order.cacheLastUpdate = order.LASTUPDATE;
					var processedItens = buildOrderItemsArray(order, sortItemsByComboOrder);
					order.processedItens = processedItens;
				}
				order.processedItens.forEach(function(items){
					items.sort(function (a, b) {
						var comparadorA = a[widget.orderItemGroupDescField] || a[widget.orderItemDescField];
						var comparadorB = b[widget.orderItemGroupDescField] || b[widget.orderItemDescField];
						if (sortItemsByComboOrder === 'true' || sortItemsByComboOrder === true) {
							comparadorA = a[widget.orderItemComboPositionField] || a[widget.orderItemGroupDescField] || a[widget.orderItemGroupDescField];
							comparadorB = b[widget.orderItemComboPositionField] || b[widget.orderItemGroupDescField];
						}
						if (comparadorA > comparadorB) {
							return 1;
						}
						if (comparadorA < comparadorB) {
							return -1;
						}
						if(a.NRITPEDIDOFOS > b.NRITPEDIDOFOS){
							return 1;
						}else{
							return -1;
						}
						// a must be equal to b
						return 0;
					});
				});
				return order.processedItens;
			};

			function buildOrderItemsArray(order, sortItemsByComboOrder = false) {
				var orderGroupedByTablePosition = groupOrderByTablePosition(order);

				let retorno = Object.keys(orderGroupedByTablePosition).map(function(tablePosition) {
					var groups = buildItemsGroups(orderGroupedByTablePosition[tablePosition]);
					var items = buildLonelyItems(orderGroupedByTablePosition[tablePosition]);
					return items.concat(groups);
				});
				// esconde o item igual ao pai
				retorno = hiddenSameItems(retorno);
				// ordena os itens pelo NRSEQITVENDAEST - controle de combo do combo
				if (sortItemsByComboOrder === 'true') {
					return sortOrderByComboPosition(retorno);
				}

				return retorno;
			}

			function hiddenSameItems(orders) {
				let localEnvironment = ApplicationContext.environment.getLocalEnvironment();
				if (widget.IDTIPOSETOR === 'P' && localEnvironment.IDHABAGRUPPROD !== 'S') {
					return orders;
				}
				if (widget.IDTIPOSETOR === 'P' && localEnvironment.SHOWCOMBOPRODUCTION !== 'S') {
					return orders;
				}
				if (widget.IDHABEXPFLUID === 'S') {
					return orders;
				}
				// verificar IDHABEXPFUID e SHOWCOMBOPRODUCTION
				if (orders && orders.length) {
					for (let i = 0; i < orders.length; i++) {
						if (orders[i] && orders[i].length) {
							for (let ii = 0; ii < orders[i].length; ii++) {
								if (orders[i][ii] && orders[i][ii].items && orders[i][ii].items.length > 0) {
									for (let iii = 0; iii < orders[i][ii].items.length; iii++) {
										let hasItems = orders[i][ii].items && orders[i][ii].items[iii];
										let condition1 = orders[i][ii].items[iii].CDPRODUTO === orders[i][ii].items[iii].CDPRODPROMOKDS;
										let condition2 = orders[i][ii].items[iii].NMPRODUTO === orders[i][ii].items[iii].NMPROMOCAO;
										let condition3 = (!orders[i][ii].items[iii].NMPROMOCAO && !orders[i][ii].NMPROMOCAO && orders[i][ii].items[iii].NMPRODUTO === orders[i][ii].NMPRODUTO);
										let condition4 = parseFloat(orders[i][ii].items[iii].QTPRODPEFOS).toFixed(3) === parseFloat(orders[i][ii].items[iii].fatherQtty).toFixed(3);
										if (!hasItems) {
											continue;
										}
										if ((condition1 || condition2 || condition3) && condition4) {
											orders[i][ii].items[iii].isVisibleItem = false;
											if (!orders[i][ii].OBSPAI) {
												orders[i][ii].OBSPAI = orders[i][ii].items[iii].TXPRODPED;
											} else if (orders[i][ii].items[iii].TXPRODPED && Array.isArray(orders[i][ii].items[iii].TXPRODPED)) {
												for (let co = 0; co < orders[i][ii].items[iii].TXPRODPED.length; co++) {
													let obsF = orders[i][ii].items[iii].TXPRODPED[co];
													if (checkObservations(obsF, orders[i][ii].OBSPAI)) {
														orders[i][ii].OBSPAI.push(obsF);
													}
												}
											} else {
												orders[i][ii].OBSPAI = '';
											}
										}
									}
								}
							}
						}
					}
				}
				return orders;
			}

			function checkObservations (obs, observations) {
				return !observations.find(e => obs.OBSERVACAO === e.OBSERVACAO);
			}
			function groupOrderByTablePosition(order) {
				var orderItems = order[widget.orderItemsField];
				return ApplicationContext.Utilities.groupArrayByProperty(orderItems, widget.orderItemPositionField);
			}
		
			function sortOrderByComboPosition(groupedItems) {
				groupedItems.forEach(group => {
					// Verificando se é necessário ordenar um grupo de itens aninhados.
					group.forEach(itemOrGroup => {
						// Se tem a propriedade 'items', trata-se de um grupo com itens aninhados.
						if (itemOrGroup.items) {
							itemOrGroup.items.sort((a, b) => parseInt(a.NRSEQITVENDAEST, 10) - parseInt(b.NRSEQITVENDAEST, 10));
						}
					});
			
					// Ordenando diretamente se a estrutura for plana (sem itens aninhados).
					// Isso se aplica se 'group' é na verdade apenas um array de itens sem 'items' aninhados.
					if (!group.some(item => item.items)) {
						group.sort((a, b) => parseInt(a.NRSEQITVENDAEST, 10) - parseInt(b.NRSEQITVENDAEST, 10));
					}

				});

				return groupedItems;
			}

			function buildLonelyItems(orders) {
				var items = [];
				if(orders) {
					items = orders.filter(function(item) {
						return !item[widget.orderItemGroupField];
					});
				}
				return items;
			}

			function buildItemsGroups(orders) {
				var groups = [], IDSITITPEDFOS = 'E', qtty= 0;
				if(orders) {
					var groupFathers = ApplicationContext.Utilities.groupArrayByProperty(orders, 'NRSEQPRODCOMKDS');
					if(groupFathers){
						Object.keys(groupFathers).forEach(function(father) {
							var fatherObj = null;
							if(father !== 'null'){
								fatherObj = {};
								var groupItems = groupFathers[father];
								groupItems.forEach(function(groupItem) {
									groupItem.items = null;
									if(groupItem.IDSITITPEDFOS == 'P'){
										fatherObj.IDSITITPEDFOS = 'P';
									}else if(IDSITITPEDFOS == 'E' && groupItem.IDSITITPEDFOS == 'F'){
										qtty++;
									}
									if(qtty == groupItems.length){
										IDSITITPEDFOS = 'F';
									}
								});
								fatherObj = Util.extend(fatherObj, groupItems[0], true);
								fatherObj.items = groupItems;
								fatherObj.isGroup = true;
								groups.push(fatherObj);
							}
						});
					}
				}
				return groups;
			}

			function hasPriorityOrders () {
				if (!widget.IDHABPRIORIDADE || widget.IDHABPRIORIDADE !== 'S') {
					return false;
				}
				if (widget.IDTIPOSETOR === 'P') {
					return false;
				}
				if (widget.dataSource.data.length === 0) {
					return false;
				}
				return !!widget.dataSource.data.find((p) => {
					let priorityProduct = p.IDPRODPRIORIDADE === true;
					let paymentCompleted = p.IDCONDPEDPAG === 'S' && p.IDSITUVENDA === 'O';
					let doesNotCondidtion = p.IDCONDPEDPAG === 'N';
					let orderCandelled = p.IDSITITPEDFOS === 'C';
					return priorityProduct && (paymentCompleted || doesNotCondidtion || orderCandelled);

				});
			}

			function removeOrders () {
				let ordersToRemove = [];
				let localEnvironment = ApplicationContext.environment.getLocalEnvironment();
				let orders = widget.dataSource.data;
				for (let i = 0; i < orders.length; i++) {
					if (!orders[i].items || orders[i].items.length === 0) {
						// armazena para ser removido posteriormente
						ordersToRemove.push(orders[i]);
					}
					// Esconde os pedidos que ainda não podem ser liberados
					let	canExpedite = widget.IDTIPOSETOR === 'E' && !orderCanCancel(orders[i]) && teste(orders[i]);

					// esconde os itens que nn podem ser expedidos ainda caso o parametro IDEXIBCONDLIB esteja habilitado
					if(localEnvironment.IDEXIBCONDLIB !== 'S' || canExpedite || widget.kdsType === 'production' ){
						orders[i].canExibOrder = true;
					} else {
						orders[i].canExibOrder = false;
						orders[i].hide = true;
					}
				}
				// remove do array os pedidos que não tem itens
				ordersToRemove.forEach((e) => {
					let idx = orders.indexOf(e);
					orders.splice(idx, 1);
				});
				return orders;
			}

			function orderCanCancel(order) {
				return order.items.every(function(item) {
					return item[widget.orderItemStatusField] === widget.orderCancelledStatus || (order.items.every(function(item) {
						return item[widget.orderItemStatusField] === widget.orderCancelledStatus;
					}));
				});
			}

			function teste (order) {
				return order.items.every(function(item) {
					return item[widget.orderItemSelectField];
				});
			}

			function removeOrdersCompleted() {
				// impede que os pedidos que ja foram concluidos voltem para a tela
				widget.completedOrders = widget.completedOrders || {};
				Object.keys(widget.completedOrders).forEach((c) => {
					let time = (moment().unix()) - (moment(widget.completedOrders[c], ('HH:mm')).unix());
					if (time > 60) {
						Reflect.deleteProperty(widget.completedOrders, c);
					}
				});
				const cpOrders = Object.values(widget.completedOrders).length;
				for (let i = 1; i < cpOrders.length + 1; i++) {
					/* jshint -W083 */
						setTimeout(() => {
							if (widget && widget.dataSource && widget.dataSource.data && widget.dataSource.data.length > 0) {
								let filteredOrders = widget.dataSource.data.filter((order) => {
									let key = order.CDSETOR + order.NRPEDIDOFOS + order.NRITPEDIDOFOS;
									return !widget.completedOrders[key];
								});
								widget.dataSource.data = filteredOrders;
							}
					}, (i * 100)/2);
				}
			}

			widget.bindPanelShortcutKeys();

		}
	};
});