function KeyboardSocketService(KeyboardEventsHandler, Utilities, ApplicationContext) {
	var self = this;
	const host = 'ws://127.0.0.1:1610';
	const useNCRKeyboard = localStorage.getItem('useNCRKeyboard') === 'true';
	var webSocket = null;
	let fieldOpen = false;
	let comandOne = 0;
	let popUp = false;
	let zoom = false;
	let focusedElement = -1;
	let last;

	const defaultKeysMap = {
		'down'      : ['p'],
		// 'left'      : ['4'],
		'zoom'      : ['b'],
		// 'right'     : ['6'],
		'up'        : ['o'],
		'dot'       : ['c'],
		'reload'    : ['i'],
		'backspace' : ['m'],
		'enter'     : ['a'],
		// 'slash'		: ['d'],
		'picklist'  : ['g'],
		'one'   	: ['e'],
		'priorityPanel': ['k']
	};

	this.startWebSocket = function() {

		if (!useNCRKeyboard) {
			return false;
		}

		webSocket = new WebSocket(host);
		webSocket.onmessage = function (event) {
			if (event && event.data) {
				handleKeyboardEvents(event.data);
			}
		};
		webSocket.onopen = function () {
			console.log('Conectado ao socket do teclado NCR');
		};
		webSocket.onclose = function () {
			console.log('conexão do socket do teclado NCR fechada');
			if (useNCRKeyboard) {
				setTimeout(() => {
					self.startWebSocket(); // reinicia o socket caso use o Teclado NCR
				}, 1000);
			}
		};
	};

	this.setPopUp = function (status) {
		comandOne = 0;
		last = 1;
		popUp = status;
	};

	this.setZoom = function (status) {
		zoom = status;
	};

	handleKeyboardEvents = function (event) {
		Object.keys(defaultKeysMap).forEach(element => {
			if (defaultKeysMap[element].includes(event)) {
				let eventName = '';
				if (comandOne === 1 && element === 'one') {
					eventName = `executeOne${Utilities.capitalizeFirstLetter(element)}Event`;
				} if (comandOne === 2 && element === 'one') {
					eventName = `executeOneOne${Utilities.capitalizeFirstLetter(element)}Event`;
					comandOne = 0;
				} else {
					eventName = `execute${Utilities.capitalizeFirstLetter(element)}Event`;
				}
				if (zoom) {
					popUp = false;
					eventName = 'executeZoom' + Utilities.capitalizeFirstLetter(element) + 'Event';
				}
				if (window.location.href.includes('initialParameters')) {
					handleSectorSelection(element);
				} else if (popUp) {
					handlePopupNavigation(element);
				} else {
					KeyboardEventsHandler[eventName]();
					if (element === 'one') {
						comandOne ++;
					} else {
						comandOne = 0;
					}
				}
			}
		});
	};

	function handleSectorSelection (e) {
		let widget = document.widget;
		if (e === 'enter' && fieldOpen == false) { // abre a lista de setores
			fieldOpen = true;
			focusedElement = -1;
			widget.getField('SELECTCDSETOR').openField();
		}
		else if (e === 'enter' && fieldOpen == true) { // seleciona setor
			fieldOpen = false;
			if (collection[focusedElement]) {
				collection[focusedElement].style.fontWeight = "normal";
				// a colletion é uma lista de setores mas o elemento clicável é uma div dentro de cada item da lista, ou seja, o neto do elemento que tem a classe zh-list-item-label
				childrenDiv = collection[focusedElement].children[0]; 
				clickableDiv = childrenDiv.children[0];
				clickableDiv.click();
				focusedElement = -1; // reseta o foco
				setTimeout(() => {
					popUp = false;
					ApplicationContext.AuthController.loginButtonOnClick(widget);
				}, 200); // entra no setor
			}
		}
 
		if (e === 'down') { // navega para baixo na lista de setores
			if (focusedElement != -1 && collection[focusedElement] && collection[focusedElement].style) {
				collection[focusedElement].style.fontWeight = "normal";
			}
			focusedElement++;
			if (focusedElement > collection.length - 1) { // se for o ultimo elemento, volta para o primeiro
				focusedElement = 0;
			}
			if (collection[focusedElement]) {
				collection[focusedElement].style.fontWeight = "bold";
				collection[focusedElement].focus();
				collection[focusedElement].scrollIntoViewIfNeeded();
			}
		}

		if (e === 'up') { // navega para cima na lista de setores
			if (focusedElement != -1 && collection[focusedElement] && collection[focusedElement].style) {
				collection[focusedElement].style.fontWeight = "normal";
			}
			focusedElement--;
			if (focusedElement < 0) { // se for o primeiro elemento, volta para o ultimo
				focusedElement = collection.length - 1;
			}
			if (collection[focusedElement]) {
				collection[focusedElement].style.fontWeight = "bold";
				collection[focusedElement].focus();
				collection[focusedElement].scrollIntoViewIfNeeded();
			}
		}
	}

	function handlePopupNavigation(e) {
		let buttons = document.querySelectorAll('.zh-alert-button');
 		if (e === 'down') {
			buttons[last].focus();
			last = last == 0 ? 1 : 0;
		} else if (e == 'up') {
			buttons[last].focus();
			last = last == 0 ? 1 : 0;
		} else if (e == 'enter') {
			last = last == 0 ? 1 : 0;
			buttons[last].click();
			last = 0;
		} else {
			last = 0;
			buttons[last].focus();
			last = last == 0 ? 1 : 0;
		}
	}
}

Configuration(function(ContextRegister) {
	ContextRegister.register('KeyboardSocketService', KeyboardSocketService);
});