function ShortcutKeyService(environment, KeyboardEventsHandler, Utilities, KeyboardSocketService) {

	var self = this;
	var keysMap = {};

	var last;

	var defaultKeysMap = {
		'down'      : ['2'        ],
		'left'      : ['4'        ],
		'zoom'      : ['YmFuYW5pbmhh','space'],
		'right'     : ['6'        ],
		'up'        : ['8'        ],
		'dot'       : ['apelidoDot'],
		'reload'    : ['**'       ],
		'backspace' : ['backspace'],
		'enter'     : ['enter'    ],
		'numlock'   : ['numlock'  ],
		'slash'		: ['slash'],
		'picklist'  : ['9'  ],
		'one'   	: ['1'  ],
		'priorityPanel': ['7']
	};

	this.bindShortcutsOrderPanel = function() {
		var localEnvironment = environment.getLocalEnvironment();
		var localKeyMap = localEnvironment.keyMapping;
		var mappedKeys = {};
		bindNumlockTurnedOffConvertion();

		// merge the backend keys whit the defalt keys and save in the front for future uses
		keysMap = Util.clone(defaultKeysMap);
		localKeyMap.forEach(function(shortcutKey) {
			if(keysMap[shortcutKey.name]){
				if(keysMap[shortcutKey.name].indexOf(shortcutKey.keyCode) == -1){
					keysMap[shortcutKey.name].push(shortcutKey.keyCode);
				}
			}else{
				keysMap[shortcutKey.name] = [shortcutKey.keyCode];
			}
		});

		bindMappedKeys(keysMap);
	};

	function bindMappedKeys() {
		unbindMappedKeys();
		KeyboardSocketService.setPopUp(false);
		KeyboardSocketService.setZoom(false);
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'execute' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	}

	this.bindZoomMappedKeys = function() {
		unbindMappedKeys();
		KeyboardSocketService.setZoom(true);
		KeyboardSocketService.setPopUp(false);
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'executeZoom' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	};

	this.bindSlashMappedKeys = function() {
		unbindMappedKeys();
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'executeSlash' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	};

	this.bindOneMappedKeys = function() {
		// unbindMappedKeys();
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'executeOne' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	};

	this.bindOneOneMappedKeys = function() {
		// unbindMappedKeys();
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'executeOneOne' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	};

	this.bindSlashEnterMappedKeys = function() {
		unbindMappedKeys();
		Object.keys(keysMap).forEach(function(eventName) {
			var callbackName = 'executeSlashEnter' + Utilities.capitalizeFirstLetter(eventName) + 'Event';
			if(KeyboardEventsHandler[callbackName]) {
				Mousetrap.bind(keysMap[eventName], Util.buildDebounceMethod(KeyboardEventsHandler[callbackName]), 'keyup');
			}
		});
	};

	this.bindPopUpKeys = function() {
		var bind = localStorage.getItem('bindPopupKeys');
		KeyboardSocketService.setPopUp(true);
		if(bind && bind == 'true') {
			unbindMappedKeys();
			document.addEventListener('keydown', handlePopupNavigation);
			last = 1;
			setTimeout(() => {
				document.querySelectorAll('.zh-alert-button')[0].focus();
			}, 500);
		}
		else {
			bindPopUpKeys();
		}
	};

	function handlePopupNavigation(e) {
		var buttons = document.querySelectorAll('.zh-alert-button');
 		if (e.key == '2' || e.key == 'ArrowDown') {
			buttons[last].focus();
			last = last == 0 ? 1 : 0;
		} else if (e.key == '8' || e.key == 'ArrowUp') {
			buttons[last].focus();
			last = last == 0 ? 1 : 0;
		} else if (e.key == 'Enter') {
			last = last == 0 ? 1 : 0;
			buttons[last].click();
			last = 0;
		} else {
			last = 0;
			buttons[last].focus();
			last = last == 0 ? 1 : 0;
		}
	}

	this.unbindPopUpKeys = function() {
		bindMappedKeys();
		KeyboardSocketService.setPopUp(false);
		document.removeEventListener('keydown', handlePopupNavigation, false);
	};

	this.unbindMappedKeys = function() {
		unbindMappedKeys();
	};

	this.bindMappedKeys = function() {
		bindMappedKeys();
	};

	function unbindMappedKeys() {
		Object.keys(keysMap).forEach(function(shortcutKey) {
			Mousetrap.unbind(keysMap[shortcutKey], 'keyup');
		});
	}

	function bindPopUpKeys() {
		Mousetrap.addKeycodes({
			37: 'left',
			38: 'up',
			39: 'right',
			40: 'down',
			104: 'up',
			98: 'down',
			100: 'left',
			102: 'right'
		});
	}

	function bindNumlockTurnedOffConvertion() {
		Mousetrap.addKeycodes({
			'144' : 'numlock',
			'109' : 'apelidoDot'    ,
			'36'  : '7'    ,
			'38'  : '8'    ,
			'33'  : '9'    ,
			'37'  : '4'    ,
			'12'  : '5'    ,
			'39'  : '6'    ,
			'194' : '.'    ,
			'35'  : '1'    ,
			'40'  : '2'    ,
			'34'  : '3'    ,
			'13'  : 'enter',
			'45'  : '0'    ,
			'46'  : ','    ,
			'110' : ',',
			'111' : 'slash',
			//o + e o * não binda no keyup sabe se lá o porque, mas com apelido funciona
			'107' : 'YmFuYW5pbmhh',
			'106' : '**',
			'104': '8',
			'97': '1',
			'98': '2',
			'100': '4',
			'102': '6',
			'101': '5',
		});
	}

}

Configuration(function(ContextRegister) {
	ContextRegister.register('ShortcutKeyService', ShortcutKeyService);
});